@@ -36,7 +36,6 @@ import subprocess
 import tarfile
 import shutil
 import datetime
-import distutils.dir_util
 import lsb_release
 from hashlib import md5
 
@@ -146,7 +145,7 @@ class Backend(dbus.service.Object):
         dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
         if session_bus:
             backend.bus = dbus.SessionBus()
-            backend.enforce_polkit = False
+            backend.enforce_polkit = True
         else:
             backend.bus = dbus.SystemBus()
         try:
@@ -374,8 +373,7 @@ class Backend(dbus.service.Object):
             if dest is not None:
                 if not os.path.isdir(dest):
                     os.makedirs(dest)
-                distutils.file_util.copy_file(fishie, dest,
-                                              verbose=1, update=0)
+                shutil.copy(fishie, dest)
 
 
     def start_sizable_progress_thread(self, input_str, mnt, w_size):
@@ -480,9 +478,7 @@ class Backend(dbus.service.Object):
                     new_name += '.zip'
                 elif os.path.exists(fishie) and tarfile.is_tarfile(fishie):
                     new_name += '.tgz'
-                distutils.file_util.copy_file(fishie,
-                                              os.path.join(dest, new_name),
-                                              verbose=1, update=0)
+                shutil.copy_file(fishie, os.path.join(dest, new_name))
 
         #If dell-recovery needs to be injected into the image
         if dell_recovery_package:
@@ -497,7 +493,7 @@ class Backend(dbus.service.Object):
                 (out, err) = call.communicate()
             else:
                 logging.debug("Adding manually included dell-recovery package, %s", dell_recovery_package)
-                distutils.file_util.copy_file(dell_recovery_package, dest)
+                shutil.copy(dell_recovery_package, dest)
 
         function = getattr(Backend, create_fn)
         function(self, assembly_tmp, version, iso, platform, no_update)
@@ -816,13 +812,22 @@ arch %s, distributor_str %s, bto_platform %s" % (bto_version, distributor, relea
                     line = "GRUB_DEFAULT=saved\n"
                 wfd.write(line)
 
+        env = os.environ
+
         #Make sure the language is set properly
         with open('/etc/default/locale','r') as rfd:
             for line in rfd.readlines():
                 if line.startswith('LANG=') and len(line.split('=')) > 1:
                     lang = line.split('=')[1].strip('\n').strip('"')
-        env = os.environ
-        env['LANG'] = lang
+                    env['LANG'] = lang
+
+        #Make sure the path is set properly
+        if 'PATH' not in env and os.path.exists('/etc/environment'):
+            with open('/etc/environment', 'r') as rfd:
+                for line in rfd.readlines():
+                    if line.startswith('PATH=') and len(line.split('=')) > 1:
+                        path = line.split('=')[1].strip('\n').strip('"')
+                        env['PATH'] = path
 
         ret = subprocess.call(['/usr/sbin/update-grub'], env=env)
         if ret is not 0:
@@ -980,6 +985,20 @@ arch %s, distributor_str %s, bto_platform %s" % (bto_version, distributor, relea
         xorrisoargs.append('-m')
         xorrisoargs.append(os.path.join('casper', old_initrd))
 
+        #Renew .disk/ubuntu_dist_channel for ubuntu-report
+        ubuntu_dist_channel = os.path.join(mntdir, '.disk', 'ubuntu_dist_channel')
+        if os.path.exists(ubuntu_dist_channel) and platform and revision:
+            xorrisoargs.append('-m')
+            xorrisoargs.append(ubuntu_dist_channel)
+            with open(os.path.join(tmpdir, '.disk', 'ubuntu_dist_channel'), 'w') as target, \
+                 open(ubuntu_dist_channel) as source:
+                for line in source:
+                    if line.startswith('canonical-oem-somerville-') and \
+                            not line.strip().endswith('+' + platform + '+' + revision):
+                        target.write(line.strip() + '+' + platform + '+' + revision + '\n')
+                    else:
+                        target.write(line)
+
         #Restore .disk/info
         info_path = os.path.join(mntdir, '.disk', 'info.recovery')
         if os.path.exists(info_path):
