@@ -92,8 +92,6 @@ def white_tree(action, whitelist, src, dst='', base=None):
 
 def _tree(action, list, src, dst, base, white):
     """Helper function for tree calls"""
-    from distutils.file_util import copy_file
-
     if base is None:
         base = src
         if not base.endswith('/'):
@@ -131,8 +129,7 @@ def _tree(action, list, src, dst, base, white):
             if action == "copy":
                 if not os.path.isdir(dst):
                     os.makedirs(dst)
-                copy_file(src_name, dst_name, preserve_mode=1,
-                          preserve_times=1, update=1, dry_run=0)
+                shutil.copy(src_name, dst_name)
                 outputs.append(dst_name)
 
             elif action == "size":
@@ -145,8 +142,8 @@ def check_family(test):
     path = '/sys/class/dmi/id/product_family'
     if not os.path.exists(path):
         return False
-    with open(path) as rfd:
-        value = rfd.readline().strip()
+    with open(path, 'rb') as rfd:
+        value = rfd.read().strip()
         if not value:
             return False
         if test.lower() in value.lower():
@@ -157,11 +154,12 @@ def check_vendor():
     """Checks to make sure that the app is running on Dell HW"""
     path = '/sys/class/dmi/id/'
     variables = ['bios_vendor', 'sys_vendor']
-    valid = ['dell', 'alienware', 'wyse', 'innotek']
+    valid = [b'dell', b'alienware', b'wyse', b'innotek']
     for var in variables:
-        if os.path.exists(path + var):
-            with open(path + var) as rfd:
-                value = rfd.readline().strip()
+        target = os.path.join(path, var)
+        if os.path.exists(target):
+            with open(target, 'rb') as rfd:
+                value = rfd.read().strip()
                 if not value:
                     return True
                 value = value.split()[0].lower()
@@ -288,7 +286,7 @@ def find_factory_partition_stats():
 
             # Only search for the recovery partition on the same disk
             device = block.get_cached_property("Device").get_bytestring().decode('utf-8')
-            if device.startswith('/dev/mmcblk') or device.startswith('/dev/nvme'):
+            if device[-1].isnumeric():
                 offset = 2
             else: # /dev/sd[a-z]
                 offset = 1
@@ -298,6 +296,11 @@ def find_factory_partition_stats():
                     if device[:-offset] in line:
                         the_same_drive = True
                         break
+                    # support dmraid corner case
+                    elif line.startswith("/dev/mapper/isw"):
+                        if transfer_dmraid_path(device)[:-2] in line:
+                            the_same_drive = True
+                            break
             if not the_same_drive:
                 continue
 
@@ -753,6 +756,25 @@ def regenerate_md5sum(root_dir,sec_dir=None):
             import syslog
             syslog.syslog("rewrite the md5sum.txt file failed with : %s" %(err))
 
+def transfer_dmraid_path(source_path):
+    """two direction change the dmraid path representive
+       sample : /dev/dm-X --> /dev/mapper/isw*
+    """
+    udisks = UDisks.Client.new_sync(None)
+    manager = udisks.get_object_manager()
+    for item in manager.get_objects():
+        block = item.get_block()
+        if not block:
+            continue
+        # Check the disk is type of dmraid
+        device_path = block.get_cached_property("Device").get_bytestring().decode('utf-8')
+        if device_path == source_path:
+            output = block.get_cached_property("Id").get_string()
+            model = output.split("-")[-1]
+            dest_path = os.path.join("/dev/mapper", model)
+            break
+    return dest_path
+
 ##                ##
 ## Common Classes ##
 ##                ##
