@@ -364,6 +364,11 @@ class Page(Plugin):
 
         #starting with 17.04, we replace the whole swap partition to swap file
         if float(release["RELEASE"]) >= 17.04:
+            try:
+                self.db.set('partman-swapfile/percentage', '50')
+                self.db.set('partman-swapfile/size', self.mem * 2048)
+            except debconf.DebconfError as err:
+                self.log(str(err))
             return True
 
         if (self.mem >= 32 or self.disk_size <= 64):
@@ -524,26 +529,68 @@ class Page(Plugin):
         udisks = UDisks.Client.new_sync(None)
         manager = udisks.get_object_manager()
         drive = None
+
+        raids = {}
+        for item in manager.get_objects():
+            mdraid = item.get_mdraid()
+            if mdraid:
+                level = mdraid.get_cached_property("Level").get_string()
+                if level in ('raid0', 'raid1', 'raid4', 'raid5', 'raid6', 'raid10'):
+                    uuid = mdraid.get_cached_property("UUID").get_string()
+                    raids[uuid] = level.upper()
+
         for item in manager.get_objects():
             loop = item.get_loop()
             block = item.get_block()
             partition = item.get_partition()
+
             if loop or \
                partition or \
                not block or \
-               not block.get_cached_property("HintPartitionable").get_boolean() or \
                block.get_cached_property("ReadOnly").get_boolean():
                 continue
 
-            #Check the disk is type of NVME SSD
+            id_type = block.get_cached_property("IdType").get_string()
+            if id_type == 'isw_raid_member':
+                continue
+
             device_path = block.get_cached_property("Device").get_bytestring().decode('utf-8')
-            if device_path.startswith('/dev/nvme'):
+
+            # Check if the disk is the type of dmraid
+            if device_path.startswith('/dev/dm'):
+                output = block.get_cached_property("Id").get_string()
+                model = output.split("-")[-1]
+                # device_path = os.path.join("/dev/mapper",model)
+                dmraid_dev_size = block.get_cached_property("Size").unpack()
+                disks.append([device_path, dmraid_dev_size, "%s (%s)" % (model, device_path)])
+                continue
+
+            # Check if the disk is the type of mdraid
+            elif device_path.startswith('/dev/md'):
+                if block.get_cached_property('Size').get_uint64() == int(0):
+                    continue
+                name = block.get_cached_property("PreferredDevice").get_bytestring().decode('utf-8').split("/")[-1]
+                uuid = block.get_cached_property("Id").get_string().split("-")[-1]
+                size = block.get_cached_property("Size").unpack()
+                model = "%s %s %i GB" % (name, raids[uuid], size / 1000000000)
+                disks.append([device_path, size, "%s (%s)" % (model, device_path)])
+                continue
+
+            # Check if the disk is the type of NVME SSD
+            elif device_path.startswith('/dev/nvme'):
                 output = block.get_cached_property("Id").get_string()
                 model = output.split("-")[-1].replace("_", " ")
                 nvme_dev_size = block.get_cached_property("Size").unpack()
                 disks.append([device_path, nvme_dev_size, "%s (%s)" % (model, device_path)])
                 continue
 
+            # Support Persistent Memory storage
+            elif device_path.startswith('/dev/pmem'):
+                pmem_dev_size = block.get_cached_property("Size").unpack()
+                model = 'Persistent Memory %i GB' % (pmem_dev_size / 1000000000)
+                disks.append([device_path, pmem_dev_size, "%s (%s)" % (model, device_path)])
+                continue
+
             drive_obj = block.get_cached_property("Drive").get_string()
             if drive_obj == '/':
                 continue
@@ -622,6 +669,11 @@ class Page(Plugin):
         #EFI install finds ESP
         self.db.set('grub-installer/bootdev', self.device)
 
+        # install GRUB if it's dmraid installation
+        if self.device.startswith("/dev/dm"):
+            grub_command = 'debconf-set partman-auto/disk %s' % magic.transfer_dmraid_path(self.device)
+            self.db.set('partman/early_command', grub_command)
+
         self.disk_size = rec_part["size_gb"]
 
         self.log("Detected device we are operating on is %s" % self.device)
@@ -658,6 +710,18 @@ class Page(Plugin):
             if self.stage == 2 and rec_part["slave"] in mount:
                 self.log("Detected RP at %s, setting to factory boot" % mount)
                 rec_type = 'factory'
+            # check if the mount point is dmraid
+            elif mount.startswith("/dev/dm") and self.stage == 2 and rec_part["slave"][:-1] in mount:
+                self.log("Detected RP at %s, setting to factory boot" % mount)
+                rec_type = 'factory'
+            # check if the mount point is mdraid
+            elif mount.startswith("/dev/md") and self.stage == 2 and rec_part["slave"][:-1] in mount:
+                self.log("Detected RP at %s, setting to factory boot" % mount)
+                rec_type = 'factory'
+            # check if the mount point is Persistent Memory
+            elif mount.startswith("/dev/pmem") and self.stage == 2 and rec_part["slave"][:-1] in mount:
+                self.log("Detected RP at %s, setting to factory boot" % mount)
+                rec_type = 'factory'
             else:
                 self.log("No (matching) RP found.  Assuming media based boot")
                 rec_type = 'dvd'
@@ -738,7 +802,7 @@ class Page(Plugin):
             self.cancel_handler()
 
         #force wyse systems to always take minimal mode (18.04+)
-        if self.db.get('dell-recovery/wyse_mode') == 'true' or magic.check_family("wyse"):
+        if self.db.get('dell-recovery/wyse_mode') == 'true' or magic.check_family(b"wyse"):
             self.preseed("ubiquity/minimal_install", "true")
 
         return (['/usr/share/ubiquity/dell-bootstrap'], [RECOVERY_TYPE_QUESTION])
@@ -881,6 +945,14 @@ manually to proceed.")
         #in mbytes
         rp_size_mb = (rp_size / 1000000) + cushion
 
+        # replace the self.device for dmraid if needed
+        # sample: /dev/dm-0 --> /dev/mapper/isw*
+        if "/dev/dm" in self.device:
+            self.device = magic.transfer_dmraid_path(self.device)
+
+        if self.device.startswith('/dev/md') and shutil.which('mdadm'):
+            misc.execute_root('mdadm', '--misc', '--action=frozen', self.device)
+
         # Build new partition table
         command = ('parted', '-s', self.device, 'mklabel', 'gpt')
         result = misc.execute_root(*command)
@@ -892,7 +964,7 @@ manually to proceed.")
         commands = [('parted', '-a', 'optimal', '-s', self.device, 'mkpart', 'primary', 'fat16', '0', str(grub_size)),
                     ('parted', '-s', self.device, 'name', '1', "'EFI System Partition'"),
                     ('parted', '-s', self.device, 'set', '1', 'boot', 'on')]
-        if '/dev/nvme' in self.device or '/dev/mmcblk' in self.device:
+        if self.device[-1].isnumeric():
             commands.append(('mkfs.msdos', self.device + 'p' + EFI_ESP_PARTITION))
             rp_part = 'p' + EFI_RP_PARTITION
             esp_part = 'p' + EFI_ESP_PARTITION
@@ -1071,6 +1143,9 @@ manually to proceed.")
             self.xml_obj.write_xml('/mnt/bto.xml')
         misc.execute_root('umount', '/mnt')
 
+        if self.device.startswith('/dev/md') and shutil.which('mdadm'):
+            misc.execute_root('mdadm', '--misc', '--action=idle', self.device)
+
         for count in range(100,0,-10):
             self.status("Restarting in %d seconds." % int(count/10), count)
             time.sleep(1)
@@ -1234,7 +1309,7 @@ class Install(InstallPlugin):
             recovery_type = 'hdd'
             #if wyse mode is on (dell-recovery/mode == 'wyse'), set the recovery_type to be 'factory'
             #as Wyse platforms will always skip the "Restore OS Linux partition" dialog
-            if self.db.get('dell-recovery/wyse_mode') == 'true' or magic.check_family("wyse"):
+            if self.db.get('dell-recovery/wyse_mode') == 'true' or magic.check_family(b"wyse"):
                 recovery_type = 'factory'
             #create 99_dell_recovery grub
             magic.create_grub_entries(self.target, recovery_type)
